/*
 * Copyright (C) 2017 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.android.settings.deviceinfo.firmwareversion;

import static com.google.common.truth.Truth.assertThat;

import android.content.Context;
import android.os.Build;

import com.android.settings.core.BasePreferenceController;
import com.android.settings.testutils.SettingsRobolectricTestRunner;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.robolectric.RuntimeEnvironment;
import org.robolectric.annotation.Config;
import org.robolectric.annotation.Implementation;
import org.robolectric.annotation.Implements;
import org.robolectric.annotation.Resetter;

import androidx.fragment.app.Fragment;
import androidx.preference.Preference;
import androidx.preference.PreferenceManager;
import androidx.preference.PreferenceScreen;

@RunWith(SettingsRobolectricTestRunner.class)
public class FirmwareVersionPreferenceControllerTest {

    private final String KEY = "firmware_key";

    @Mock
    private Fragment mFragment;

    private Preference mPreference;
    private PreferenceScreen mScreen;
    private FirmwareVersionPreferenceController mController;

    @Before
    public void setup() {
        MockitoAnnotations.initMocks(this);
        final Context context = RuntimeEnvironment.application;
        final PreferenceManager preferenceManager = new PreferenceManager(context);
        mController = new FirmwareVersionPreferenceController(context, KEY);
        mController.setHost(mFragment);
        mPreference = new Preference(context);
        mPreference.setKey(KEY);
        mScreen = preferenceManager.createPreferenceScreen(context);
        mScreen.addPreference(mPreference);
    }

    @After
    public void tearDown() {
        ShadowFirmwareVersionDialogFragment.reset();
    }

    @Test
    public void firmwareVersion_shouldAlwaysBeShown() {
        assertThat(mController.getAvailabilityStatus()).isEqualTo(
                BasePreferenceController.AVAILABLE);
    }

    @Test
    public void updatePreference_shouldSetSummaryToBuildNumber() {
        mController.updateState(mPreference);

        assertThat(mPreference.getSummary()).isEqualTo(Build.VERSION.RELEASE);
    }

    @Test
    @Config(shadows = ShadowFirmwareVersionDialogFragment.class)
    public void handlePreferenceTreeClick_samePreferenceKey_shouldStartDialogFragment() {
        final boolean result = mController.handlePreferenceTreeClick(mPreference);

        assertThat(ShadowFirmwareVersionDialogFragment.isShowing).isTrue();
        assertThat(result).isTrue();
    }

    @Test
    public void handlePreferenceTreeClick_unknownPreferenceKey_shouldDoNothingAndReturnFalse() {
        mPreference.setKey("foobar");

        final boolean result = mController.handlePreferenceTreeClick(mPreference);

        assertThat(ShadowFirmwareVersionDialogFragment.isShowing).isFalse();
        assertThat(result).isFalse();
    }

    @Implements(FirmwareVersionDialogFragment.class)
    public static class ShadowFirmwareVersionDialogFragment {

        private static boolean isShowing = false;

        @Implementation
        public static void show(Fragment fragemnt) {
            isShowing = true;
        }

        @Resetter
        public static void reset() {
            isShowing = false;
        }
    }
}
